@@ -8,11 +8,12 @@ from chromadb.errors import (
 )
 import uuid
 import time
-import os
-import itertools
+import numpy.typing as npt
 import json
-from typing import Optional, Sequence, List, Tuple, cast
+from typing import Dict, Optional, Sequence, List, Tuple, cast
 import clickhouse_connect
+from clickhouse_connect.driver.client import Client
+from clickhouse_connect import common
 
 COLLECTION_TABLE_SCHEMA = [{"uuid": "UUID"}, {"name": "String"}, {"metadata": "String"}]
 
@@ -34,7 +35,7 @@ def db_array_schema_to_clickhouse_schema(table_schema):
     return return_str
 
 
-def db_schema_to_keys() -> list[str]:
+def db_schema_to_keys() -> List[str]:
     keys = []
     for element in EMBEDDING_TABLE_SCHEMA:
         keys.append(list(element.keys())[0])
@@ -51,20 +52,17 @@ class Clickhouse(DB):
         self._settings = settings
 
     def _init_conn(self):
+        common.set_setting("autogenerate_session_id", False)
         self._conn = clickhouse_connect.get_client(
             host=self._settings.clickhouse_host, port=int(self._settings.clickhouse_port)
         )
-        self._conn.query(f"""SET allow_experimental_lightweight_delete = 1;""")
-        self._conn.query(
-            f"""SET mutations_sync = 1;"""
-        )  # https://clickhouse.com/docs/en/operations/settings/settings/#mutations_sync
         self._create_table_collections(self._conn)
         self._create_table_embeddings(self._conn)
 
-    def _get_conn(self):
+    def _get_conn(self) -> Client:
         if self._conn is None:
             self._init_conn()
-        return self._conn
+        return self._conn  # type: ignore because we know it's not None
 
     def _create_table_collections(self, conn):
         conn.command(
@@ -86,7 +84,7 @@ class Clickhouse(DB):
     def persist(self):
         raise NotImplementedError("Clickhouse is a persistent database, this method is not needed")
 
-    def get_collection_uuid_from_name(self, name):
+    def get_collection_uuid_from_name(self, name: str) -> str:
         res = self._get_conn().query(
             f"""
             SELECT uuid FROM collections WHERE name = '{name}'
@@ -94,8 +92,14 @@ class Clickhouse(DB):
         )
         return res.result_rows[0][0]
 
-    def _create_where_clause(self, collection_uuid, ids=None, where={}, where_document={}):
-        where_clauses = []
+    def _create_where_clause(
+        self,
+        collection_uuid: str,
+        ids: Optional[List[str]] = None,
+        where: Where = {},
+        where_document: WhereDocument = {},
+    ):
+        where_clauses: List[str] = []
         self._format_where(where, where_clauses)
         if len(where_document) > 0:
             where_document_clauses = []
@@ -106,43 +110,36 @@ class Clickhouse(DB):
             where_clauses.append(f" id IN {tuple(ids)}")
 
         where_clauses.append(f"collection_uuid = '{collection_uuid}'")
-        # We know that where_clauses is not empty, so force typechecker
-        where = " AND ".join(cast(List[str], where_clauses))
-        where = f"WHERE {where}"
-        return where
+        where_str = " AND ".join(where_clauses)
+        where_str = f"WHERE {where_str}"
+        return where_str
 
     #
     #  COLLECTION METHODS
     #
-    def create_collection(self, name, metadata=None):
-        if metadata is None:
-            metadata = {}
-
+    def create_collection(
+        self, name: str, metadata: Optional[Dict] = None, get_or_create: bool = False
+    ) -> Sequence:
         # poor man's unique constraint
-        checkname = (
-            self._get_conn()
-            .query(
-                f"""
-            SELECT * FROM collections WHERE name = '{name}'
-        """
-            )
-            .result_rows
-        )
+        dupe_check = self.get_collection(name)
 
-        if len(checkname) > 0:
-            raise Exception("Collection already exists with that name")
+        if len(dupe_check) > 0:
+            if get_or_create:
+                print(f"collection with name {name} already exists, returning existing collection")
+                return dupe_check
+            else:
+                raise ValueError(f"Collection with name {name} already exists")
 
         collection_uuid = uuid.uuid4()
-        data_to_insert = []
-        data_to_insert.append([collection_uuid, name, json.dumps(metadata)])
+        data_to_insert = [[collection_uuid, name, json.dumps(metadata)]]
 
         self._get_conn().insert(
             "collections", data_to_insert, column_names=["uuid", "name", "metadata"]
         )
-        return collection_uuid
+        return [[collection_uuid, name, metadata]]
 
-    def get_collection(self, name):
-        return (
+    def get_collection(self, name: str):
+        res = (
             self._get_conn()
             .query(
                 f"""
@@ -151,31 +148,35 @@ class Clickhouse(DB):
             )
             .result_rows
         )
+        # json.loads the metadata
+        return [[x[0], x[1], json.loads(x[2])] for x in res]
 
-    def list_collections(self) -> Sequence[Sequence[str]]:
-        return self._get_conn().query(f"""SELECT * FROM collections""").result_rows
+    def list_collections(self) -> Sequence:
+        res = self._get_conn().query(f"""SELECT * FROM collections""").result_rows
+        return [[x[0], x[1], json.loads(x[2])] for x in res]
 
-    def update_collection(self, current_name, new_name, new_metadata):
+    def update_collection(
+        self, current_name: str, new_name: Optional[str] = None, new_metadata: Optional[Dict] = None
+    ):
         if new_name is None:
             new_name = current_name
         if new_metadata is None:
-            new_metadata = self.get_collection(current_name)[0]
+            new_metadata = self.get_collection(current_name)[0][2]
 
         return self._get_conn().command(
             f"""
 
-         ALTER TABLE 
-            collections 
+         ALTER TABLE
+            collections
          UPDATE
-            metadata = {new_metadata}, 
+            metadata = '{json.dumps(new_metadata)}',
             name = '{new_name}'
-         WHERE 
-
+         WHERE
             name = '{current_name}'
          """
         )
 
-    def delete_collection(self, name):
+    def delete_collection(self, name: str):
         collection_uuid = self.get_collection_uuid_from_name(name)
         self._get_conn().command(
             f"""
@@ -190,7 +191,6 @@ class Clickhouse(DB):
         )
 
         self._idx.delete_index(collection_uuid)
-        return True
 
     #
     #  ITEM METHODS
@@ -237,10 +237,10 @@ class Clickhouse(DB):
                 parameters[f"d{i}"] = documents[i]
 
             update_statement = f"""
-            UPDATE 
+            UPDATE
                 {",".join(update_fields)}
             WHERE
-                id = {{i{i}:String}} AND 
+                id = {{i{i}:String}} AND
                 collection_uuid = '{collection_uuid}'{"" if i == len(ids) - 1 else ","}
             """
             updates.append(update_statement)
@@ -259,7 +259,7 @@ class Clickhouse(DB):
         # Verify all IDs exist
         existing_items = self.get(collection_uuid=collection_uuid, ids=ids)
         if len(existing_items) != len(ids):
-            raise ValueError("Some of the supplied ids for update were not found")
+            raise ValueError(f"Could not find {len(ids) - len(existing_items)} items for update")
 
         # Update the db
         self._update(collection_uuid, ids, embeddings, metadatas, documents)
@@ -270,7 +270,7 @@ class Clickhouse(DB):
             self._idx.delete_from_index(collection_uuid, update_uuids)
             self._idx.add_incremental(collection_uuid, update_uuids, embeddings)
 
-    def _get(self, where={}, columns: Optional[list] = None):
+    def _get(self, where={}, columns: Optional[List] = None):
         select_columns = db_schema_to_keys() if columns is None else columns
         val = (
             self._get_conn()
@@ -317,7 +317,9 @@ class Clickhouse(DB):
                         return result.append(f" JSONExtractString(metadata,'{key}') = '{operand}'")
                     return result.append(f" JSONExtractFloat(metadata,'{key}') = {operand}")
                 else:
-                    raise ValueError(f"Operator {operator} not supported")
+                    raise ValueError(
+                        f"Expected one of $gt, $lt, $gte, $lte, $ne, $eq, got {operator}"
+                    )
             elif type(value) == list:
                 all_subresults = []
                 for subwhere in value:
@@ -329,7 +331,7 @@ class Clickhouse(DB):
                 elif key == "$and":
                     result.append(f"({' AND '.join(all_subresults)})")
                 else:
-                    raise ValueError(f"Operator {key} not supported with a list of where clauses")
+                    raise ValueError(f"Expected one of $or, $and, got {key}")
 
     def _format_where_document(self, where_document, results):
         operator = list(where_document.keys())[0]
@@ -346,20 +348,20 @@ class Clickhouse(DB):
             if operator == "$and":
                 results.append(f"({' AND '.join(all_subresults)})")
         else:
-            raise ValueError(f"Operator {operator} not supported")
+            raise ValueError(f"Epected one of $contains, $and, $or, got {operator}")
 
     def get(
         self,
-        where={},
-        collection_name=None,
-        collection_uuid=None,
-        ids=None,
-        sort=None,
-        limit=None,
-        offset=None,
-        where_document={},
-        columns: Optional[List] = None,
-    ):
+        where: Where = {},
+        collection_name: Optional[str] = None,
+        collection_uuid: Optional[str] = None,
+        ids: Optional[IDs] = None,
+        sort: Optional[str] = None,
+        limit: Optional[int] = None,
+        offset: Optional[int] = None,
+        where_document: WhereDocument = {},
+        columns: Optional[List[str]] = None,
+    ) -> Sequence:
         if collection_name == None and collection_uuid == None:
             raise TypeError("Arguments collection_name and collection_uuid cannot both be None")
 
@@ -368,36 +370,38 @@ class Clickhouse(DB):
 
         s3 = time.time()
 
-        where = self._create_where_clause(
-            collection_uuid, ids=ids, where=where, where_document=where_document
+        where_str = self._create_where_clause(
+            # collection_uuid must be defined at this point, cast it for typechecker
+            cast(str, collection_uuid),
+            ids=ids,
+            where=where,
+            where_document=where_document,
         )
 
         if sort is not None:
-            where += f" ORDER BY {sort}"
+            where_str += f" ORDER BY {sort}"
         else:
-            where += f" ORDER BY collection_uuid"  # stable ordering
+            where_str += f" ORDER BY collection_uuid"  # stable ordering
 
         if limit is not None or isinstance(limit, int):
-            where += f" LIMIT {limit}"
+            where_str += f" LIMIT {limit}"
 
         if offset is not None or isinstance(offset, int):
-            where += f" OFFSET {offset}"
+            where_str += f" OFFSET {offset}"
 
-        val = self._get(where=where, columns=columns)
+        val = self._get(where=where_str, columns=columns)
 
         return val
 
-    def _count(self, collection_uuid):
-        where_string = ""
-        if collection_uuid is not None:
-            where_string = f"WHERE collection_uuid = '{collection_uuid}'"
+    def _count(self, collection_uuid: str):
+        where_string = f"WHERE collection_uuid = '{collection_uuid}'"
         return self._get_conn().query(f"SELECT COUNT() FROM embeddings {where_string}").result_rows
 
-    def count(self, collection_name):
+    def count(self, collection_name: str):
         collection_uuid = self.get_collection_uuid_from_name(collection_name)
         return self._count(collection_uuid=collection_uuid)[0][0]
 
-    def _delete(self, where_str=None):
+    def _delete(self, where_str: Optional[str] = None):
         deleted_uuids = (
             self._get_conn().query(f"""SELECT uuid FROM embeddings {where_str}""").result_rows
         )
@@ -411,7 +415,12 @@ class Clickhouse(DB):
         return [res[0] for res in deleted_uuids] if len(deleted_uuids) > 0 else []
 
     def delete(
-        self, where={}, collection_name=None, collection_uuid=None, ids=None, where_document={}
+        self,
+        where: Where = {},
+        collection_name: Optional[str] = None,
+        collection_uuid: Optional[str] = None,
+        ids: Optional[IDs] = None,
+        where_document: WhereDocument = {},
     ):
         if collection_name == None and collection_uuid == None:
             raise TypeError("Arguments collection_name and collection_uuid cannot both be None")
@@ -421,20 +430,23 @@ class Clickhouse(DB):
 
         s3 = time.time()
         where_str = self._create_where_clause(
-            collection_uuid, ids=ids, where=where, where_document=where_document
+            # collection_uuid must be defined at this point, cast it for typechecker
+            cast(str, collection_uuid),
+            ids=ids,
+            where=where,
+            where_document=where_document,
         )
 
         deleted_uuids = self._delete(where_str)
 
-        # if len(where) == 1:
-        #     self._idx.delete(collection_uuid)
         self._idx.delete_from_index(collection_uuid, deleted_uuids)
 
         return deleted_uuids
 
-    def get_by_ids(self, ids: list, columns: Optional[list] = None):
+    def get_by_ids(self, ids: list, columns: Optional[List] = None):
+        columns = columns + ["uuid"] if columns else ["uuid"]
         select_columns = db_schema_to_keys() if columns is None else columns
-        return (
+        response = (
             self._get_conn()
             .query(
                 f"""
@@ -444,6 +456,11 @@ class Clickhouse(DB):
             .result_rows
         )
 
+        # sort db results by the order of the uuids
+        response = sorted(response, key=lambda obj: ids.index(obj[len(columns) - 1]))
+
+        return response
+
     def get_nearest_neighbors(
         self,
         where: Where,
@@ -452,7 +469,7 @@ class Clickhouse(DB):
         n_results: int,
         collection_name=None,
         collection_uuid=None,
-    ) -> Tuple[List[List[uuid.UUID]], List[List[float]]]:
+    ) -> Tuple[List[List[uuid.UUID]], npt.NDArray]:
 
         # Either the collection name or the collection uuid must be provided
         if collection_name == None and collection_uuid == None:
@@ -495,7 +512,7 @@ class Clickhouse(DB):
 
         return uuids, distances
 
-    def create_index(self, collection_uuid) -> None:
+    def create_index(self, collection_uuid: str):
         """Create an index for a collection_uuid and optionally scoped to a dataset.
         Args:
             collection_uuid (str): The collection_uuid to create an index for
